from __future__ import unicode_literals

import os
from bs4 import BeautifulSoup
from django.db import models, IntegrityError

from bikestats.scraper import Scraper


class Make(models.Model):
    name = models.CharField(max_length=200)
    description = models.TextField(default='')
    last_modified = models.DateTimeField(auto_now=True)

    class Meta:
        unique_together = ('name',)


class Model(models.Model):
    name = models.CharField(max_length=200)
    make = models.ForeignKey(Make)
    years = models.CharField(max_length=64)
    description = models.TextField(default='')
    last_modified = models.DateTimeField(auto_now=True)

    class Meta:
        unique_together = ('name', 'make', 'years')


class Stat(models.Model):
    name = models.CharField(max_length=64)
    value = models.CharField(max_length=64)
    model = models.ForeignKey(Model)

    class Meta:
        unique_together = ('name', 'model')


def parse_all(scraped_root_path):
    """ Parse all bike information and stick in database. Current naive-mode *shrugs*

    Args:
        scraped_root_path: Path to the scraped directory of the website
    """
    soup = BeautifulSoup(open(os.path.join(scraped_root_path, 'index.html')).read(), "html.parser")
    for name_make, make_href in Scraper.parse_makes(soup):
        make, created_make = Make.objects.get_or_create(name=name_make)
        make_soup = BeautifulSoup(open(os.path.join(scraped_root_path, make_href)).read(), "html.parser")
        models, description_make, pages = Scraper.parse_make(make_soup, name=name_make)
        for page in pages:
            path = os.path.join(scraped_root_path, 'bikes', page)
            try:
                page_soup = BeautifulSoup(open(path).read(), "html.parser")
                _models, _d, _p = Scraper.parse_make(page_soup, name=name_make)
                models.extend(_models)
            except:
                print 'Failed to parse page with path: ' + path
        for name_model, model_href, years in models:
            model, created_model = Model.objects.get_or_create(name=name_model, make=make, years=years)
            model_soup = BeautifulSoup(open(os.path.join(scraped_root_path, 'bikes', model_href)).read(), "html.parser")
            name_model_2, description_model, stats = Scraper.parse_model(model_soup, make_name=name_make, model_name=name_model)
            for name_stat, value in stats:
                try:
                    Stat.objects.update_or_create(name=name_stat, value=value, model=model)
                except IntegrityError:
                    print 'IntegrityError for make/model: ' + name_make + '/' + name_model + " stat: " + name_stat



from django.contrib import admin
from django.contrib.auth.views import logout
from django.conf.urls import url


admin.autodiscover()


urlpatterns = [
    # url(r'^admin/', include(admin.site.urls)),
]



import re


class Scraper(object):
    """ We scrape the website and store the information in the models. We save the urls
    and the previously modified time, so that if anything is updated, we may update our
    resources. We also don't want to peg the site too much, by being good webizens. """

    @staticmethod
    def parse_model(soup, make_name='unknown make', model_name='unknown model'):
        """ Parse a model of bike """
        for strip_tag in ['script', 'style']:
            for s in soup(strip_tag):
                s.extract()
        # gather stats
        stats = []
        for i, table_row in enumerate(soup.select('table[cellspacing=1] tr')):
            try:
                tds = table_row.select('td')
                model_name = tds[0].text.strip()
                value = tds[1].text.strip()
                stats.append([model_name, value])
                table_row.extract()
            except:
                print 'Failed to parse table_row: ' + str(i) + ' for make/model: ' + make_name + '/' + model_name
        if stats and stats[0]:
            model_name = stats[0][1]
        # extract description
        description = ''
        try:
            parent_element = soup.select('table[cellspacing=1]')[0]
            description_element = parent_element.parent.parent.parent.parent.parent
            description = unicode(description_element).strip()
        except:
            print 'Failed to parse description from make/model: ' + make_name + '/' + model_name
        return (model_name, description, stats)

    @staticmethod
    def parse_makes(soup):
        """ Parse all the makes (manufacturers) from the left bar """
        for make_anchor in soup.select('.leftMenu_container > a'):
            make = make_anchor.text.strip()
            href = make_anchor['href']
            yield [make, href]

    @staticmethod
    def parse_make(soup, recursive=False, name='unknown make'):
        """ Parse make (manufacturer) page

        Args:
            recursive: Boolean representing if each child page should also be parsed
        """
        for strip_tag in ['script', 'style']:
            for s in soup(strip_tag):
                s.extract()
        # grab models from each page
        models = list(Scraper.parse_make_models(soup, name))
        pages = Scraper.parse_make_pages(soup)
        # scrape make description
        description = ''
        try:
            soup_element = soup.select('div > table > tr div')
            description = unicode(soup_element[1]).replace('\r', '').replace('\t', '').strip()
        except:
            print 'Failed to parse description for make: ' + name
        return (models, description, pages)

    @staticmethod
    def parse_make_pages(soup):
        """ Generate list of pages, e.g. 1, 2, 3, 4 """
        pages = []
        for anchor in soup.select('p > font > a'):
            href = anchor['href']
            if 'model' not in href and 'http' not in href:
                pages.append(href)
        return set(pages)

    @staticmethod
    def parse_make_models(soup, make_name='unknown make'):
        """ Parse all the make's models on a given page. """
        for i, model_row in enumerate(soup.select('div > table > tr div > table > tr')):
            try:
                anchor = model_row.select('a')[0]
                name = anchor.text.replace("\n","").replace("\r","").replace("\t","").strip()
                href = anchor['href']
                # check if model has date information in another td
                years = -1
                if len(model_row.select('td')) > 1:
                    years = model_row.select('td')[1].text.strip().rstrip('-')
                if 'http' not in href:
                    yield [name, href, years]
            except:
                print 'Failed to parse model_row: ' + str(i) + ' for make: ' + make_name



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-08 16:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Make',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(default='')),
                ('last_modified', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='Model',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('years', models.CharField(max_length=64)),
                ('description', models.TextField(default='')),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('make', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bikestats.Make')),
            ],
        ),
        migrations.CreateModel(
            name='Stat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('value', models.CharField(max_length=64)),
                ('model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bikestats.Model')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='make',
            unique_together=set([('name',)]),
        ),
        migrations.AlterUniqueTogether(
            name='stat',
            unique_together=set([('name', 'model')]),
        ),
        migrations.AlterUniqueTogether(
            name='model',
            unique_together=set([('name', 'make', 'years')]),
        ),
    ]



import os
import unittest
from bs4 import BeautifulSoup
from django.test import TestCase

from bikestats.models import Make, Model, Stat, parse_all
from bikestats.scraper import Scraper


TEST_DIR = os.path.join(os.path.dirname(__file__), '../../www.motorcyclespecs.co.za')


class TestScraper(TestCase):
    """ Test our scraped data. Note: you must have scraped the entire website before this test! """
    def test_scrape_root(self):
        parse_all(TEST_DIR)
        # TODO some sort of functional test, not just a count of makes
        self.assertEqual(132, Make.objects.count())
        self.assertEqual(4290, Model.objects.count())
        self.assertEqual(26482, Stat.objects.count())


if __name__ == '__main':
    unittest.main()



import os
from bs4 import BeautifulSoup
from django.test import TestCase

from bikestats.scraper import Scraper


TEST_DIR = os.path.abspath(os.path.dirname(__file__))


class TestScraper(TestCase):
    def test_scrape_root(self):
        with open(os.path.join(TEST_DIR, r'http:_www.motorcyclespecs.co.za_.html')) as f:
            soup = BeautifulSoup(f.read(), "html.parser")
            makes = list(Scraper.parse_makes(soup))
            self.assertEqual(132, len(makes))

    def test_scrape_make(self):
        with open(os.path.join(TEST_DIR, r'http:_www.motorcyclespecs.co.za_bikes_Ducati.htm')) as f:
            soup = BeautifulSoup(f.read(), "html.parser")
            models, description, pages = Scraper.parse_make(soup)
            self.assertEqual(101, len(models))

    def test_scrape_model(self):
        with open(os.path.join(TEST_DIR, r'http:_www.motorcyclespecs.co.za_model_ducati_ducati_monster_1200s%2015.htm')) as f:
            soup = BeautifulSoup(f.read(), "html.parser")
            name, description, stats = Scraper.parse_model(soup)
            self.assertEqual('Ducati Monster 1200S', name)


if __name__ == '__main':
    unittest.main()



from pip.req import parse_requirements
from setuptools import setup, find_packages
from bikestats import __version__ as version

try:
    requirements = [
        str(req.req) for req in parse_requirements('requirements.txt')
    ]
except TypeError:
    requirements = [
        str(req.req) for req in parse_requirements('requirements.txt', session=False)
    ]

setup(
    name='bikestats',
    version=version,
    description=('Scrape and save information from motorcyclespecs.co.za'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='bike motorcycle scraper meta',
    author='Jon Robison',
    author_email='narfman0@gmail.com',
    url='https://github.com/narfman0/bikestats',
    license='LICENSE',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=requirements,
    test_suite='tests',
)
